<?php

namespace onespace\tools\activeApi\components\clients\teams\text;

/**
 * Set the text size of a text block in a teams message
 *
 * @see https://adaptivecards.io/schemas/adaptive-card.json#FontSize
 */
enum TextSize: string {
    case UNSET = "UNSET";
    case DEFAULT = "default";
    case SMALL = "small";
    case MEDIUM = "medium";
    case LARGE = "large";
    case EXTRA_LARGE = "extraLarge";
}
