<?php

namespace onespace\tools\activeApi\components\clients\teams\actions;

use onespace\tools\activeApi\components\clients\teams\TeamsMessageComponent;

/**
 * Common class for various action tooling to be included in a teams message
 */
abstract class TeamsActions implements TeamsMessageComponent {
}
