<?php

namespace onespace\tools\activeApi\components\clients\teams\text;

/**
 * Set a font type for a text block in a teams message
 *
 * @see https://adaptivecards.io/schemas/adaptive-card.json#FontType
 */
enum TextFontType: string {
    case UNSET = "UNSET";
    case DEFAULT = "default";
    case MONOSPACE = "monospace";
}
