<?php

namespace onespace\tools\activeApi\components\clients\teams\actions;

/**
 * Available actions a message can include on a teams message.
 */
enum ActionOption: string {
    case OPEN_URL = "Action.OpenUrl";
}
