<?php

namespace onespace\tools\activeApi\components\clients\teams\text;

/**
 * Set the weight of a text block in a teams message
 *
 * @see https://adaptivecards.io/schemas/adaptive-card.json#FontWeight
 */
enum TextWeight: string {
    case UNSET = "UNSET";
    case DEFAULT = "default";
    case LIGHTER = "lighter";
    case BOLDER = "bolder";
}
